from django.db import models


class User(models.Model):
    nick_name = models.CharField(max_length=256)

    def __str__(self):
        return self.nick_name


class Tag(models.Model):
    title = models.CharField(max_length=256)

    def __str__(self):
        return self.title


class Post(models.Model):
    title = models.CharField(max_length=256)
    summary = models.TextField()
    updated = models.DateTimeField()
    author = models.ForeignKey(User, related_name='posts')
    tags = models.ManyToManyField(Tag, related_name='posts')

    def __str__(self):
        return self.title



import os
from django.test import TestCase
from .models import User, Tag, Post
from parser_mapper import ParserMapper, ObjectMapping, FieldMapping

TESTS_DIR = os.path.dirname(os.path.abspath(__file__))


class TestParser(TestCase):
    def test_can_get_model_without_any_nested_models(self):
        self.assertEqual(User.objects.count(), 0)

        mapping = ObjectMapping(
            None, 'tests.User',
            (FieldMapping('nick_name', 'author'),)
        )
        source = os.path.join(TESTS_DIR, 'habr_source.xml')
        parser_mapper = ParserMapper(source, mapping)

        parser_mapper.put_to_models()
        self.assertEqual(User.objects.count(), 20)

        # after restart not add duplicates to db
        parser_mapper.put_to_models()
        self.assertEqual(User.objects.count(), 20)

    def test_can_get_many_nested_models(self):
        self.assertEqual(Tag.objects.count(), 0)

        mapping = ObjectMapping(
            'tags', 'tests.Tag',
            (FieldMapping('title', 'term'),)
        )
        source = os.path.join(TESTS_DIR, 'habr_source.xml')
        parser_mapper = ParserMapper(source, mapping)
        parser_mapper.put_to_models()

        self.assertEqual(Tag.objects.count(), 129)
        self.assertTrue(Tag.objects.filter(title="positive technologies").exists())

    def test_can_get_model_with_nested_models(self):
        self.assertEqual(Tag.objects.count(), 0)
        self.assertEqual(User.objects.count(), 0)
        self.assertEqual(Post.objects.count(), 0)

        mapping = ObjectMapping(
            None, 'tests.Post',
            (
                FieldMapping('title', 'title'),
                FieldMapping('summary', 'description'),
                FieldMapping('tags',
                    ObjectMapping(
                        'tags', 'tests.Tag',
                        (FieldMapping('title', 'term'),)
                    )
                ),
                FieldMapping('author',
                    ObjectMapping(
                        None, 'tests.User',
                        (FieldMapping('nick_name', 'author'),)
                    )
                ),
                FieldMapping('updated', 'published')
            )
        )

        source = os.path.join(TESTS_DIR, 'habr_source.xml')
        parser_mapper = ParserMapper(source, mapping)
        parser_mapper.put_to_models()

        # verify result
        self.assertEqual(Tag.objects.count(), 129)
        self.assertTrue(Tag.objects.filter(title="positive technologies").exists())

        self.assertEqual(User.objects.count(), 20)

        self.assertEqual(Post.objects.count(), 20)
        self.assertNotEqual(Post.objects.first().title, '')
        self.assertNotEqual(Post.objects.first().author, None)
        tags_in_post_qty = len(
            Post.objects.values_list('tags__title', flat=True).distinct()
        )
        self.assertEqual(tags_in_post_qty, 129)

    def test_correct_parse_reddit(self):
        mapping = ObjectMapping(
            None, 'tests.Post',
            (
                FieldMapping('title', 'title'),
                FieldMapping('author',
                    ObjectMapping(
                        None, 'tests.User',
                        (FieldMapping('nick_name', 'author'),)
                    )
                ),
                FieldMapping('updated', 'updated')
            )
        )
        source = os.path.join(TESTS_DIR, 'reddit_source.xml')
        parser_mapper = ParserMapper(source, mapping)
        parser_mapper.put_to_models()

        self.assertEqual(Post.objects.count(), 25)
        self.assertEqual(User.objects.count(), 21)
        self.assertEqual(Tag.objects.count(), 0)


#!/usr/bin/env python

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


import os

here = os.path.dirname(os.path.abspath(__file__))
f = open(os.path.join(here, 'README.md'))
long_description = f.read().strip()
f.close()


setup(
    name='nosql2django',
    version='0.1.1',
    author='Egor Poderiagin',
    author_email='egor@crazyrussian.pro',
    url='https://github.com/Akay7/nosql2django',
    description='Allow convert nosql source to django relational DB structure',
    packages=find_packages(),
    long_description=long_description,
    keywords='django rss atom feeds parser sql nosql-to-sql',
    zip_safe=False,
    install_requires=[
        'Django>=1.8.0',
        'feedparser>=5.2.1',
        'python-dateutil>=2.6.0',
    ],
    test_suite='runtests.runtests',
    include_package_data=True,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



from collections import namedtuple
from functools import reduce
import feedparser
from dateutil import parser as time_parser
from django.apps import apps

ObjectMapping = namedtuple('RelatedField', ('base_path', 'model', 'fields'))
FieldMapping = namedtuple('Field', ('name', 'mapping'))


def deep_get(deep_dict, path):
    if isinstance(path, str):
        path = (path,)

    return reduce(
        lambda d, key: d.get(key) if isinstance(d, dict) else None,
        path, deep_dict
    )


class ParserMapper:
    def __init__(self, source, mapping):
        self.source = source
        self.mapping = mapping

    @staticmethod
    def save_to_db(model_text_id, parsed_values):
        """save to db and return saved object"""
        Model = apps.get_model(model_text_id)

        # normalise values and separate to m2m, simple
        simple_fields = {}
        many2many_fields = {}
        for field, value in parsed_values.items():
            if (Model._meta.get_field(
                    field).get_internal_type() == 'ManyToManyField'):
                many2many_fields[field] = value
            elif (Model._meta.get_field(
                    field).get_internal_type() == 'DateTimeField'):
                simple_fields[field] = time_parser.parse(value)

            else:
                simple_fields[field] = value

        # ToDo: add unique identify parameter to field
        # ToDo: allow unique identify m2m field
        model, created = Model.objects.get_or_create(**simple_fields)

        for field, value in many2many_fields.items():
            setattr(model, field, value)
        model.save()

        return model

    @staticmethod
    def parse_obj(mapping, obj):
        def _parse_single_obj(mapping, obj):
            parsed_values = {}
            for field in mapping.fields:
                if isinstance(field.mapping, ObjectMapping):
                    value = ParserMapper.parse_obj(
                        field.mapping, obj[field.mapping.base_path] if field.mapping.base_path else obj
                    )
                else:
                    value = deep_get(obj, field.mapping)
                parsed_values[field.name] = value

            return ParserMapper.save_to_db(mapping.model, parsed_values)

        if isinstance(obj, list):
            return [_parse_single_obj(mapping, i) for i in obj]
        return _parse_single_obj(mapping, obj)

    def put_to_models(self):
        feed = feedparser.parse(self.source)

        for e in feed['entries']:
            ParserMapper.parse_obj(
                self.mapping,
                e[self.mapping.base_path] if self.mapping.base_path else e
            )
